import type { ArticleStore } from "../domain/articleStore";
import { useCallback, useEffect } from "react";
import { getArticle } from "article/data/articleAPIService";
import { Article } from "article/domain/articleEntity";

function useArticleViewModel(
  store: ArticleStore,
  fetchArticleUseCase: (
    fetchArticleCallback: (id: string) => Promise<Article | null>,
    setArticle: ArticleStore["setArticle"],
    id: string,
  ) => Promise<Article | null>,
  getArticleUseCase: (
    getArticle: ArticleStore["getArticle"],
    setArticle: ArticleStore["setArticle"],
    id: Article["id"],
  ) => Promise<Article | null>,
  id: string,
) {
  let article: Article | null | undefined;

  const _getArticle = useCallback(
    (id: string) => {
      getArticleUseCase(store.getArticle, store.setArticle, id).then(async (value) => {
        if (value == null) {
          article = await fetchArticleUseCase(getArticle, store.setArticle, id);
          return;
        }
        article = value;
      });
    },
    [store.setArticle]
  );

  useEffect(() => {
    _getArticle(id);
  }, [article]);

  return {
    article: store.article,
    shouldShowLoading: typeof store.article === "undefined" || store.isLoading,
    hasError: store.hasError,
  };
}

export { useArticleViewModel };
